#!/usr/bin/perl
use strict;
use warnings;
use Cwd qw{realpath};
use File::Basename;
my $configdir="/etc/summoner";
my $diskconfigfile="$configdir/disk/config";
my $packagelist="";
my %conf;
my %disks; 
my %profile;

my ($profilefile)=@ARGV;
if($profilefile eq "" ){die "Usage:summoner.pl <profile>";};
$profilefile=realpath($profilefile);
my $baseprofiledir=dirname($profilefile);
print "Reading profile file  $profilefile in $baseprofiledir\n";
open(PROFILEFILED,"< $profilefile") or die("Can not find profile $profilefile\n");
while (my $line = <PROFILEFILED> )
{
    chomp($line);
    $line =~ /^\#/ and next;
    $line =~ /^$/ and next;
    my ($varname, $varvalue) = split('=',$line);
    $profile{$varname}=$varvalue;
}
close(PROFILEFILED);

if(not exists($profile{'NAME'}))
    {die("Machine name not defined in profile\n")};
    
$profile{'NAME'}=~s/\"//;
$profile{'NAME'}=~s/\;//;

if($profile{'NAME'} eq "")
    {die("Machine name is empty line\n")};
    
print "Summoning machine $profile{'NAME'}\n";

open (CONF,"< ".$diskconfigfile) or die $!;
while (my $line = <CONF> )
{
    chomp($line);
    $line =~ /^\#/ and next;
    $line =~ /^$/ and next;
    my ($varname, $varvalue) = split('=',$line);
    $conf{$varname}=$varvalue;
}
close(CONF);

#print $conf{'DEFFILEDIR'}." ".$conf{'DEFVOLUMEGROUP'}."\n";
sub read_disks 
{  
    my ($profiledir)=@_;
    my %disk;
    if(substr($profiledir,0,1) ne '/')
	{ print("Found relative path $profiledir\n");
	  $profiledir="$baseprofiledir/$profiledir";
	}
    print "Reading disks information from $profiledir\n";
    if(open(DPARENT,"< $profiledir/parent"))
    {
	my $line=<DPARENT>;
	close(DPARENT);
	$line=~s/\n//;
	print "Found parent dir $line\n";
	read_disks($line);
    }
    opendir (PRODIR,$profiledir) or die ("can not open profile dir\n");
    while( my $file = readdir PRODIR )
    {
        $file =~ /^\./ and next;
        $file =~ 'options' and next;
	$file =~ 'parent' and next;
	
        open(FDISK,"< ".$profiledir."/".$file) or die ("can not open file $file\n");
	while (my $line = <FDISK> ){
	    chomp($line);
	    $line =~ /^\#/ and next;
	    $line =~ /^$/ and next;
    
            my ($varname, $varvalue) = split('=',$line);
	    $disk{$varname}=$varvalue;
	}
        close(FDISK);
	$disk{'DEVNAME'}=$file;
    
	if(exists($disk{'NAME'}))
    	    { $disk{'DISKNAME'}=$profile{'NAME'}."-".$disk{'NAME'} }
	else
	    { $disk{'DISKNAME'}=$profile{'NAME'}."-".$disk{'DEVNAME'} }
	print("Disk name $disk{'DISKNAME'}\n");
        if(not exists($disk{'MNTPNT'}))
    	    { $disk{'MNTPNT'}="none" }
	
        if(not exists($disk{'FSOPTS'}))
    	    { $disk{'FSOPTS'}="" }

        while(my ($dkey,$dvalue) =each(%disk))
	{
	    $disks{$disk{'MNTPNT'}}{$dkey}=$dvalue;
        }
    }
    closedir(PRODIR);
}

if(exists($profile{'DISKPROFILE'}))
{
    read_disks($profile{'DISKPROFILE'});
}
else
{
    read_disks("$configdir/disk/default");
}

for my $disk (keys %disks)
{
    print "Creating disk $disk\n";
    if($disks{$disk}{'TYPE'} eq 'file')
	{
	    $disks{$disk}{'DEVPATH'}="$conf{'DEFFILEDIR'}/$disks{$disk}{'DISKNAME'}";
	    print "makeing file $disks{$disk}{'DEVPATH'}  fs=$disks{$disk}{'FS'} fsopts=$disks{$disk}{'FSOPTS'} mount point=$disks{$disk}{'MNTPNT'}\n";
	
	my $size=$disks{$disk}{'SIZE'};
	$size=~s/k//;
	$size=~s/M/k/;
	$size=~s/G/M/;
	print "creating $size blocks";
	system("dd if=/dev/zero of=$disks{$disk}{'DEVPATH'} bs=1024 count=$size");
	
    }
    elsif ($disks{$disk}{'TYPE'} eq 'lvm') 
    {
	$disks{$disk}{'DEVPATH'}="/dev/$conf{'DEFVOLUMEGROUP'}/$disks{$disk}{'DISKNAME'}";
	print "makeing lvm VG= LV=$disks{$disk}{'DISKNAME'} dev=$disks{$disk}{'DEVNAME'} size=$disks{$disk}{'SIZE'} fs=$disks{$disk}{'FS'} fsopts=$disks{$disk}{'FSOPTS'} mount point=$disks{$disk}{'MNTPNT'}\n";
	system("lvcreate -L $disks{$disk}{'SIZE'} -n $disks{$disk}{'DISKNAME'} $conf{'DEFVOLUMEGROUP'}");
     }     	
    else 
    {
	print "Unknow disk type\n"
    }
    
    if($disks{$disk}{'FS'} eq "swap" )
    {
     system("mkswap $disks{$disk}{'DEVPATH'}");
    }
    else
    {
    my $cmd="mkfs.$disks{$disk}{'FS'} -F $disks{$disk}{'FSOPTS'} $disks{$disk}{'DEVPATH'}";
    print "run:$cmd\n";
    system($cmd);
    }
}

my $tmpmntdir=`mktemp -d`;
$tmpmntdir=~s/\n//;
print "Temp mount dir created $tmpmntdir\n";
if(not exists($disks{'/'})) {die "Not found root filesystem\n"};
print "Mounting root file system $disks{'/'}{'DEVPATH'}\n";
if($disks{'/'}{'TYPE'} eq 'file')
{ system("mount -o loop $disks{'/'}{'DEVPATH'} $tmpmntdir"); }
else 
{ system("mount  $disks{'/'}{'DEVPATH'} $tmpmntdir"); }

for my $fs ( keys %disks ){
    $fs eq '/' and next;
    $fs eq 'swap' and next;
    $fs eq 'none' and next;
    print "Mounting $fs from $disks{$fs}{'DEVPATH'}\n";
    system("mkdir $tmpmntdir/$fs\n");
    if($disks{$fs}{'TYPE'} eq 'file')
    { system("mount -o loop $disks{$fs}{'DEVPATH'} $tmpmntdir/$fs\n"); }
    else 
    { system("mount  $disks{$fs}{'DEVPATH'} $tmpmntdir/$fs\n"); }

}
print "Initilazing RPM database\n";
system("mkdir -p $tmpmntdir/var/lib/rpm");
system("rpm -r $tmpmntdir --initdb");
print "Installing base packages\n";

if(not exists($profile{'REPOPROFILE'}))
{$profile{'REPOPROFILE'}="$configdir/repo/default";}

my $sourceslist="$profile{'REPOPROFILE'}/sources.list";
if(substr($sourceslist,0,1) ne '/')
	{ 
	    print("Found relative path $sourceslist\n");
	    $sourceslist="$baseprofiledir/$sourceslist";
	}

print("Using source list from $sourceslist\n");
system("apt-get -o RPM::RootDir=\"$tmpmntdir\" -o Dir::Etc::SourceList=\"$sourceslist\" update");
print "generating package list\n";

sub read_package_list {
    my ($packageprofile)=@_;
    if(substr($packageprofile,0,1) ne '/')
    { 
	print("Found relative path $packageprofile\n");
	$packageprofile="$baseprofiledir/$packageprofile";
    }
	
    if(open(DPARENT,"< $packageprofile/parent"))
    {
	my $line=<DPARENT>;
	close(DPARENT);
	$line=~s/\n//;
	print "Found parent profile $line\n";
	read_package_list($line);
    }

    open(PKGLIST,"< $packageprofile/packages.list") or die("Not found package list $packageprofile/packages.list\n");
    print("Reading package from $packageprofile/packages.list\n");
    while(my $line =<PKGLIST>)
    {
	chomp($line);
	$line =~ /^\#/ and next;
	$line =~ /^$/ and next;
	$packagelist="$packagelist $line";
    } 
    close(PKGLIST);
}
if(not exists($profile{'PACKAGEPROFILE'})){
    read_package_list("$configdir/package/default/");
}
else
{
    read_package_list($profile{'PACKAGEPROFILE'})
}
print "$packagelist\n";
system("apt-get -o RPM::RootDir=\"$tmpmntdir\" -o Dir::Etc::SourceList=\"$sourceslist\"  install -y $packagelist\n");
sleep 1;
while(`pidof rpmdb` ne '') {sleep 1;} #Awating for RPM rebuild database
sleep 2;
print "Making config files\n";

print "Making fstab\n";
open(FSTAB,">>$tmpmntdir/etc/fstab") or die("Can`t open $tmpmntdir/etc/fstab\n");
for my $fs ( keys %disks ){
    print("Writing $fs to fstab\n");
    print FSTAB "/dev/$disks{$fs}{'DEVNAME'}\t$disks{$fs}{'MNTPNT'}\t$disks{$fs}{'FS'}\tdefaults\t1\t1\n";  
}
close(FSTAB);

print("Making xen config\n");
open(XEN,">/etc/xen/$profile{'NAME'}") or die("Can`t open xen config /etc/xen/$profile{'NAME'}\n");
print XEN "kernel=\"$conf{'KERNEL'}\"\n";
print XEN "memory=$profile{'MEMORY'}\n";
print XEN "name=\"$profile{'NAME'}\"\n";
print XEN "vif = [";
my $if=0;
if(not exists($profile{'NETIF'})){$profile{'NETIF'}=0}
while($if<$profile{'NETIF'}){
    if($if != 0){print XEN ",";};
    print XEN sprintf("'mac=00:16:3e:%x:%x:%x'",rand(256),rand(256),rand(256));
    ++$if;
}
print XEN "]\n";

print XEN "disk = [";
my $xtfirst=1;
for my $fs ( keys %disks ){
    if($xtfirst == 0){print XEN ",";};
    $xtfirst=0;
    if($disks{$fs}{'TYPE'} eq 'lvm' ) {
        print XEN "'phy:$disks{$fs}{'DEVPATH'},$disks{$fs}{'DEVNAME'},w'";
     }
     else
     {
        print XEN "'tap:aio:$disks{$fs}{'DEVPATH'},$disks{$fs}{'DEVNAME'},w'";
     }
}
print XEN "]\n";
print XEN "root=\"/dev/$disks{'/'}{'DEVNAME'} ro\"\n";
close(XEN);

for my $fs ( keys %disks ){
    $fs eq '/' and next;
    $fs eq 'swap' and next;
    $fs eq 'none' and next;
    print "Unmounting $fs from $disks{$fs}{'DEVPATH'}\n";
    system("umount $tmpmntdir/$fs");
}

print "Umount root filesystem\n";
system("umount $tmpmntdir");
system("rmdir $tmpmntdir");
print "Removed temp dir\n";